/****************************************************************************
** Meta object code from reading C++ file 'categoryentriesmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../arianna-26.07.90/src/categoryentriesmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'categoryentriesmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9BookEntryE_t {};
} // unnamed namespace

template <> constexpr inline auto BookEntry::qt_create_metaobjectdata<qt_meta_tag_ZN9BookEntryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BookEntry",
        "filename",
        "filetitle",
        "title",
        "genres",
        "keywords",
        "characters",
        "series",
        "seriesNumbers",
        "seriesVolumes",
        "author",
        "rights",
        "publisher",
        "created",
        "lastOpenedTime",
        "currentLocation",
        "currentProgress",
        "thumbnail",
        "description",
        "comment",
        "tags",
        "locations",
        "identifier",
        "source",
        "language",
        "rating"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
        // property 'filename'
        QtMocHelpers::PropertyData<QString>(1, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'filetitle'
        QtMocHelpers::PropertyData<QString>(2, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'title'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'genres'
        QtMocHelpers::PropertyData<QStringList>(4, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'keywords'
        QtMocHelpers::PropertyData<QStringList>(5, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'characters'
        QtMocHelpers::PropertyData<QStringList>(6, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'series'
        QtMocHelpers::PropertyData<QStringList>(7, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'seriesNumbers'
        QtMocHelpers::PropertyData<QStringList>(8, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'seriesVolumes'
        QtMocHelpers::PropertyData<QStringList>(9, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'author'
        QtMocHelpers::PropertyData<QStringList>(10, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'rights'
        QtMocHelpers::PropertyData<QString>(11, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'publisher'
        QtMocHelpers::PropertyData<QString>(12, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'created'
        QtMocHelpers::PropertyData<QDateTime>(13, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'lastOpenedTime'
        QtMocHelpers::PropertyData<QDateTime>(14, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'currentLocation'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'currentProgress'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'thumbnail'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'description'
        QtMocHelpers::PropertyData<QStringList>(18, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'comment'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'tags'
        QtMocHelpers::PropertyData<QStringList>(20, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'locations'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'identifier'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'source'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'language'
        QtMocHelpers::PropertyData<QString>(24, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Constant),
        // property 'rating'
        QtMocHelpers::PropertyData<int>(25, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Constant),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BookEntry, qt_meta_tag_ZN9BookEntryE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BookEntry::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9BookEntryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9BookEntryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9BookEntryE_t>.metaTypes,
    nullptr
} };

void BookEntry::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = reinterpret_cast<BookEntry *>(_o);
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->filename; break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->filetitle; break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->title; break;
        case 3: *reinterpret_cast<QStringList*>(_v) = _t->genres; break;
        case 4: *reinterpret_cast<QStringList*>(_v) = _t->keywords; break;
        case 5: *reinterpret_cast<QStringList*>(_v) = _t->characters; break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->series; break;
        case 7: *reinterpret_cast<QStringList*>(_v) = _t->seriesNumbers; break;
        case 8: *reinterpret_cast<QStringList*>(_v) = _t->seriesVolumes; break;
        case 9: *reinterpret_cast<QStringList*>(_v) = _t->author; break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->rights; break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->publisher; break;
        case 12: *reinterpret_cast<QDateTime*>(_v) = _t->created; break;
        case 13: *reinterpret_cast<QDateTime*>(_v) = _t->lastOpenedTime; break;
        case 14: *reinterpret_cast<QString*>(_v) = _t->currentLocation; break;
        case 15: *reinterpret_cast<int*>(_v) = _t->currentProgress; break;
        case 16: *reinterpret_cast<QString*>(_v) = _t->thumbnail; break;
        case 17: *reinterpret_cast<QStringList*>(_v) = _t->description; break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->comment; break;
        case 19: *reinterpret_cast<QStringList*>(_v) = _t->tags; break;
        case 20: *reinterpret_cast<QString*>(_v) = _t->locations; break;
        case 21: *reinterpret_cast<QString*>(_v) = _t->identifier; break;
        case 22: *reinterpret_cast<QString*>(_v) = _t->source; break;
        case 23: *reinterpret_cast<QString*>(_v) = _t->language; break;
        case 24: *reinterpret_cast<int*>(_v) = _t->rating; break;
        default: break;
        }
    }
}
namespace {
struct qt_meta_tag_ZN20CategoryEntriesModelE_t {};
} // unnamed namespace

template <> constexpr inline auto CategoryEntriesModel::qt_create_metaobjectdata<qt_meta_tag_ZN20CategoryEntriesModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CategoryEntriesModel",
        "QML.Element",
        "auto",
        "countChanged",
        "",
        "entryDataUpdated",
        "BookEntry",
        "entry",
        "entryRemoved",
        "entryDataChanged",
        "entryRemove",
        "append",
        "CategoryEntriesModel::Roles",
        "compareRole",
        "clear",
        "getBookEntry",
        "std::optional<BookEntry>",
        "index",
        "bookFromFile",
        "filename",
        "indexOfFile",
        "indexIsBook",
        "count",
        "Roles",
        "UnknownRole",
        "FilenameRole",
        "FiletitleRole",
        "TitleRole",
        "SeriesRole",
        "SeriesNumbersRole",
        "SeriesVolumesRole",
        "AuthorRole",
        "PublisherRole",
        "CreatedRole",
        "LastOpenedTimeRole",
        "CurrentLocationRole",
        "CurrentProgressRole",
        "CategoryEntriesModelRole",
        "CategoryEntryCountRole",
        "ThumbnailRole",
        "DescriptionRole",
        "CommentRole",
        "TagsRole",
        "RatingRole",
        "GenreRole",
        "KeywordRole",
        "CharacterRole",
        "LocationsRole",
        "EntryRole"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'countChanged'
        QtMocHelpers::SignalData<void()>(3, 4, QMC::AccessPublic, QMetaType::Void),
        // Signal 'entryDataUpdated'
        QtMocHelpers::SignalData<void(const BookEntry &)>(5, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'entryRemoved'
        QtMocHelpers::SignalData<void(const BookEntry &)>(8, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'entryDataChanged'
        QtMocHelpers::SlotData<void(const BookEntry &)>(9, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'entryRemove'
        QtMocHelpers::SlotData<void(const BookEntry &)>(10, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<void(const BookEntry &, CategoryEntriesModel::Roles)>(11, 4, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 12, 13 },
        }}),
        // Method 'append'
        QtMocHelpers::MethodData<void(const BookEntry &)>(11, 4, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Method 'clear'
        QtMocHelpers::MethodData<void()>(14, 4, QMC::AccessPublic, QMetaType::Void),
        // Method 'getBookEntry'
        QtMocHelpers::MethodData<std::optional<BookEntry>(int)>(15, 4, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::Int, 17 },
        }}),
        // Method 'bookFromFile'
        QtMocHelpers::MethodData<std::optional<BookEntry>(const QString &)>(18, 4, QMC::AccessPublic, 0x80000000 | 16, {{
            { QMetaType::QString, 19 },
        }}),
        // Method 'indexOfFile'
        QtMocHelpers::MethodData<int(const QString &)>(20, 4, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 19 },
        }}),
        // Method 'indexIsBook'
        QtMocHelpers::MethodData<bool(int)>(21, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'count'
        QtMocHelpers::PropertyData<int>(22, QMetaType::Int, QMC::DefaultPropertyFlags, 0),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Roles'
        QtMocHelpers::EnumData<enum Roles>(23, 23, QMC::EnumFlags{}).add({
            {   24, Roles::UnknownRole },
            {   25, Roles::FilenameRole },
            {   26, Roles::FiletitleRole },
            {   27, Roles::TitleRole },
            {   28, Roles::SeriesRole },
            {   29, Roles::SeriesNumbersRole },
            {   30, Roles::SeriesVolumesRole },
            {   31, Roles::AuthorRole },
            {   32, Roles::PublisherRole },
            {   33, Roles::CreatedRole },
            {   34, Roles::LastOpenedTimeRole },
            {   35, Roles::CurrentLocationRole },
            {   36, Roles::CurrentProgressRole },
            {   37, Roles::CategoryEntriesModelRole },
            {   38, Roles::CategoryEntryCountRole },
            {   39, Roles::ThumbnailRole },
            {   40, Roles::DescriptionRole },
            {   41, Roles::CommentRole },
            {   42, Roles::TagsRole },
            {   43, Roles::RatingRole },
            {   44, Roles::GenreRole },
            {   45, Roles::KeywordRole },
            {   46, Roles::CharacterRole },
            {   47, Roles::LocationsRole },
            {   48, Roles::EntryRole },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<CategoryEntriesModel, void>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject CategoryEntriesModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20CategoryEntriesModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20CategoryEntriesModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20CategoryEntriesModelE_t>.metaTypes,
    nullptr
} };

void CategoryEntriesModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CategoryEntriesModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->countChanged(); break;
        case 1: _t->entryDataUpdated((*reinterpret_cast<std::add_pointer_t<BookEntry>>(_a[1]))); break;
        case 2: _t->entryRemoved((*reinterpret_cast<std::add_pointer_t<BookEntry>>(_a[1]))); break;
        case 3: _t->entryDataChanged((*reinterpret_cast<std::add_pointer_t<BookEntry>>(_a[1]))); break;
        case 4: _t->entryRemove((*reinterpret_cast<std::add_pointer_t<BookEntry>>(_a[1]))); break;
        case 5: _t->append((*reinterpret_cast<std::add_pointer_t<BookEntry>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<CategoryEntriesModel::Roles>>(_a[2]))); break;
        case 6: _t->append((*reinterpret_cast<std::add_pointer_t<BookEntry>>(_a[1]))); break;
        case 7: _t->clear(); break;
        case 8: { std::optional<BookEntry> _r = _t->getBookEntry((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<std::optional<BookEntry>*>(_a[0]) = std::move(_r); }  break;
        case 9: { std::optional<BookEntry> _r = _t->bookFromFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<std::optional<BookEntry>*>(_a[0]) = std::move(_r); }  break;
        case 10: { int _r = _t->indexOfFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->indexIsBook((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CategoryEntriesModel::*)()>(_a, &CategoryEntriesModel::countChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CategoryEntriesModel::*)(const BookEntry & )>(_a, &CategoryEntriesModel::entryDataUpdated, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CategoryEntriesModel::*)(const BookEntry & )>(_a, &CategoryEntriesModel::entryRemoved, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->count(); break;
        default: break;
        }
    }
}

const QMetaObject *CategoryEntriesModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CategoryEntriesModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20CategoryEntriesModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int CategoryEntriesModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void CategoryEntriesModel::countChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CategoryEntriesModel::entryDataUpdated(const BookEntry & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void CategoryEntriesModel::entryRemoved(const BookEntry & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
